#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# Qiang Du <du.qiang@gmail.com>
# 01/07/2015

import etherbone as eb
import sys

class EB:
    """ access a Etherbone device. """

    def __init__(self, ip):
        """ Initialize EB class.
        ip  -- IP address of Etherbone device
        """
        self.port_ip = 'udp/' + ip
        self.write_cnt = 0
        self.read_cnt = 0
        self.open()

    def open(self):
        self.socket = eb.Socket()
        self.socket.__enter__()

        self.device = eb.Device(self.socket, self.port_ip)
        self.device.__enter__()

        self.cycle = eb.Cycle(self.device, 0, 0)
        self.cycle.__enter__()

    def close(self):
        self.cycle.close()
        self.device.close()
        self.socket.close()

    def write(self, addr, val):
        self.device.write(addr, val)

    def read(self, addr):
        return self.device.read(addr)

class EI2C:

  """Access a VME crate through Telnet."""

  def __init__(self, ip, user, pwd):

    """Initialize EI2C class.

    The arguments to init the class are mandatory:
    ip   -- IP address of VME crate
    user -- username to login to VME crate
    pwd  -- password to login to VME crate
    """

    self.ip   = ip
    self.user = user
    self.pwd  = pwd
    self.write_cnt = 0
    self.read_cnt  = 0

    if (ei2cdefine.DUMP):
        self.f = open("dump.txt", 'w')

  def open(self):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM);
    try:
      s.connect((self.ip, 23))
      s.recv(30)
    except (socket.gaierror, socket.error) as e:
      raise BadHostnameError
    s.send(self.user + "\r\n")
    if "password" not in s.recv(30):
      raise BadUsernameError
    s.send(self.pwd + "\r\n")
    if "Wrong password" in s.recv(30):
      raise BadPasswordError
    self.handle = s

  def close(self):
    self.handle.close()
    if (ei2cdefine.DUMP):
        self.f.close()

  def write(self, slot, addr, val):
    self.write_cnt += 1
    #reg = addr/4 + 1
    cmd = "writereg %d %x %x\r\n" % (slot, addr, val)
    if (ei2cdefine.DUMP):
        self.f.write(cmd);
    self.handle.send(cmd)
    self._strip_resp(self.handle.recv(30))

  def read(self, slot, addr):
    self.read_cnt += 1
    #reg = addr/4 + 1
    cmd = "readreg %d %x\r\n" % (slot, addr)
    if (ei2cdefine.DUMP):
        self.f.write(cmd);
    self.handle.send(cmd)
    resp = self._strip_resp(self.handle.recv(30))
    return resp

  def writemregs(self, slot, addr, val):
    self.write_cnt += 1
    #reg = addr/4 + 1
    #print '%02x' % sl
    cmd = "writemregs %d %x %s\r\n" % (slot, addr, ' '.join(format(b,'x') for b in val))
    if (ei2cdefine.DUMP):
        self.f.write(cmd);
    #print cmd
    self.handle.send(cmd)
    self._strip_resp(self.handle.recv(30))

  def readmregs(self, slot, addr, nrregs):
    self.read_cnt += 1
    #reg = addr/4 + 1
    cmd = "readmregs %d %x %d\r\n" % (slot, addr, nrregs)
    if (ei2cdefine.DUMP):
        self.f.write(cmd);
    self.handle.send(cmd)
    resp = self._strip_resp(self.handle.recv(30))
    return resp

  def _strip_resp(self, msg):

      """Strip useful message from SysMon response.

      The response string from the SysMon is stripped of unnecessary characters
      and a "useful" string is returned. In case one of the error messages appear,
      the function raises one of the EI2C exceptions."""

      #print msg.splitlines()
      msg = msg.splitlines()[1]
      if (ei2cdefine.DUMP):
          self.f.write(msg + "\n")

      if "Not Ack" in msg:
        raise NAckError

      if "Invalid register number!" in msg:
        raise RegNumberError

      if "Bad command!" in msg:
        raise InvalidCmdError

      if "I2C Timer expired" in msg:
        raise TimerExpiredError

      if "Invalid slot number!" in msg:
        raise SlotError

      if "Read Data:" in msg:
        msg = msg.split(" ")[3]
        msg = int(msg,16)

      return msg

# MAIN "FUNCTION"
if __name__ == "__main__":
    address = 0x20810

    if (0):
        with eb.Socket() as socket:
            with eb.Device(socket, 'udp/rflab2') as device:
                with eb.Cycle(device, 0, 0) as cycle:
                    data = cycle.read(address)
        print hex(data.value)

    print 'EB test'
    ebi = EB('rflab2')
    print hex(ebi.read(address).value)
